@@ -1,10 +1,12 @@
 package main
 
 import (
+	"context"
 	"errors"
 	"fmt"
 	"log"
 	"os"
+	"time"
 
 	"github.com/ddddddO/gtree"
 	"github.com/fatih/color"
@@ -37,14 +39,25 @@ func main() {
 			Usage:       "set this option when the markdown indent is 4 spaces.",
 			DefaultText: "tab spaces",
 		},
+	}
+
+	outputFlags := []cli.Flag{
 		&cli.BoolFlag{
 			Name:    "massive",
 			Aliases: []string{"m"},
 			Usage:   "set this option when there are very many blocks of markdown.",
 		},
-	}
-
-	outputFlags := []cli.Flag{
+		&cli.DurationFlag{
+			Name:    "massive-timeout",
+			Aliases: []string{"mt"},
+			Usage:   "set this option if you want to set a timeout.",
+			Action: func(ctx *cli.Context, v time.Duration) error {
+				if v <= 0 {
+					return errors.New("the timeout value should be greater than 0.")
+				}
+				return nil
+			},
+		},
 		&cli.BoolFlag{
 			Name:        "json",
 			Aliases:     []string{"j"},
@@ -144,7 +157,7 @@ func main() {
 			{
 				Name:    "web",
 				Aliases: []string{"w", "www"},
-				Usage:   "Opens \"Tree Maker\" in your browser. If it doesn't open, it will display the url.",
+				Usage:   "Opens \"Tree Maker\" in your browser and shows the URL in terminal.",
 				Flags:   webFlags,
 				Before:  notExistArgs,
 				Action:  actionWeb,
@@ -191,10 +204,16 @@ func actionOutput(c *cli.Context) error {
 	if err != nil {
 		return exitErrOpts(err)
 	}
-	options := []gtree.Option{oi, oo}
+	var om gtree.Option
 	if c.Bool("massive") {
-		options = append(options, gtree.WithMassive())
+		om = gtree.WithMassive(context.Background())
 	}
+	if c.Duration("massive-timeout") > 0 {
+		ctx, cancel := context.WithTimeout(context.Background(), c.Duration("massive-timeout"))
+		defer cancel()
+		om = gtree.WithMassive(ctx)
+	}
+	options := []gtree.Option{oi, oo, om}
 
 	markdownPath := c.Path("file")
 	if isInputStdin(markdownPath) {
@@ -243,7 +262,7 @@ func actionMkdir(c *cli.Context) error {
 	oe := gtree.WithFileExtensions(c.StringSlice("extension"))
 	options := []gtree.Option{oi, oe}
 	if c.Bool("massive") {
-		options = append(options, gtree.WithMassive())
+		options = append(options, gtree.WithMassive(context.Background()))
 	}
 
 	if c.Bool("dry-run") {
@@ -275,11 +294,8 @@ func actionTemplate(c *cli.Context) error {
 const treeMakerURL = "https://ddddddo.github.io/gtree/"
 
 func actionWeb(c *cli.Context) error {
-	if err := openWeb(treeMakerURL, c.Bool("wsl")); err != nil {
-		fmt.Println(treeMakerURL)
-		return nil
-	}
-
+	_ = openWeb(treeMakerURL, c.Bool("wsl"))
+	fmt.Printf("See: %s\n", treeMakerURL)
 	return nil
 }
 
