@@ -3,16 +3,36 @@ package gtree_test
 import (
 	"bufio"
 	"bytes"
+	"context"
 	"errors"
 	"fmt"
 	"io"
 	"os"
 	"strings"
 	"testing"
+	"time"
 
 	"github.com/ddddddO/gtree"
+	tu "github.com/ddddddO/gtree/testutil"
+	"go.uber.org/goleak"
 )
 
+func TestMain(m *testing.M) {
+	goleak.VerifyTestMain(m)
+}
+
+func TestOutput_detecting_goroutineleak(t *testing.T) {
+	ctx, cancel := context.WithTimeout(context.Background(), time.Duration(500*time.Millisecond))
+	defer cancel()
+	w := io.Discard
+	r := strings.NewReader(tu.TwentyThousandRoots)
+	if gotErr := gtree.Output(w, r, gtree.WithMassive(ctx)); gotErr != nil {
+		if gotErr != context.DeadlineExceeded {
+			t.Errorf("\ngotErr: \n%v\nwantErr: \n%v", gotErr, context.DeadlineExceeded)
+		}
+	}
+}
+
 type in struct {
 	input   io.Reader
 	options []gtree.Option
@@ -316,6 +336,54 @@ a
 ├── e
 │   └── o
 └── g
+`, "\n"),
+				err: nil,
+			},
+		},
+		{
+			name: "case(succeeded/multi root/massive)",
+			in: in{
+				input: strings.NewReader(strings.TrimSpace(`
+- a
+	- i
+		- u
+			- k
+			- kk
+		- t
+	- e
+		- o
+	- g
+- a
+	- i
+		- u
+			- k
+			- kk
+		- t
+	- e
+		- o
+	- g`)),
+				options: []gtree.Option{gtree.WithMassive(context.Background())},
+			},
+			out: out{
+				output: strings.TrimPrefix(`
+a
+├── i
+│   ├── u
+│   │   ├── k
+│   │   └── kk
+│   └── t
+├── e
+│   └── o
+└── g
+a
+├── i
+│   ├── u
+│   │   ├── k
+│   │   └── kk
+│   └── t
+├── e
+│   └── o
+└── g
 `, "\n"),
 				err: nil,
 			},
@@ -357,10 +425,23 @@ a
 				err:    bufio.ErrTooLong,
 			},
 		},
+		{
+			name: "case(massive/bufio.Scanner err)",
+			in: in{
+				input: strings.NewReader(strings.TrimSpace(fmt.Sprintf(`
+- a
+	- %s`, strings.Repeat("a", 64*1024)))),
+				options: []gtree.Option{gtree.WithMassive(context.Background())},
+			},
+			out: out{
+				output: "",
+				err:    bufio.ErrTooLong,
+			},
+		},
 		{
 			name: "case(succeeded/input markdown file)",
 			in: in{
-				input: prepareMarkdownFile(t)},
+				input: tu.PrepareMarkdownFile(t)},
 			out: out{
 				output: strings.TrimPrefix(`
 a
@@ -454,6 +535,21 @@ a
 				err:    errors.New("invalid node name: b/c"),
 			},
 		},
+		{
+			name: "case(input format error)",
+			in: in{
+				input: strings.NewReader(strings.TrimSpace(`
+- a
+	 - b`)),
+				options: []gtree.Option{
+					gtree.WithDryRun(),
+				},
+			},
+			out: out{
+				output: "",
+				err:    gtree.ExportErrIncorrectFormat("	 - b"),
+			},
+		},
 		{
 			name: "case(succeeded/tab on the way)",
 			in: in{
@@ -478,7 +574,7 @@ a	prev tab
 	- b
 		- c`)),
 				options: []gtree.Option{
-					gtree.WithMassive(),
+					gtree.WithMassive(context.Background()),
 				},
 			},
 			out: out{
@@ -501,7 +597,7 @@ a
 		- c
 	- y`)),
 				options: []gtree.Option{
-					gtree.WithMassive(),
+					gtree.WithMassive(context.Background()),
 					gtree.WithDryRun(),
 					gtree.WithFileExtensions([]string{"c"}),
 				},
@@ -528,7 +624,7 @@ a
 	- b
 		- c`)),
 				options: []gtree.Option{
-					gtree.WithMassive(),
+					gtree.WithMassive(context.Background()),
 					gtree.WithEncodeJSON(),
 				},
 			},
@@ -546,7 +642,7 @@ a
 	- b
 		- c`)),
 				options: []gtree.Option{
-					gtree.WithMassive(),
+					gtree.WithMassive(context.Background()),
 					gtree.WithEncodeYAML(),
 				},
 			},
@@ -571,7 +667,7 @@ children:
 	- b
 		- c`)),
 				options: []gtree.Option{
-					gtree.WithMassive(),
+					gtree.WithMassive(context.Background()),
 					gtree.WithEncodeTOML(),
 				},
 			},
@@ -616,15 +712,6 @@ children = []
 	}
 }
 
-func prepareMarkdownFile(t *testing.T) *os.File {
-	const testfilepath = "./testdata/sample6.md"
-	file, err := os.Open(testfilepath)
-	if err != nil {
-		t.Fatal(err)
-	}
-	return file
-}
-
 func TestOutput_encodeJSON(t *testing.T) {
 	tests := []struct {
 		name string
