@@ -8,10 +8,7 @@ import (
 
 // Output outputs a tree to w with r as Markdown format input.
 func Output(w io.Writer, r io.Reader, options ...Option) error {
-	conf, err := newConfig(options)
-	if err != nil {
-		return err
-	}
+	conf := newConfig(options)
 
 	rg := newRootGenerator(r, conf.space)
 	roots, err := rg.generate()
